/* global L, I, O, W, U */

"use strict";

var P = {
    implementation: {},
    ui: {},

    mouseSensitivity: 1,
    touchSensitivity: 0.25,
    images: 10,
    imageStart: 3,
    imageEnd: 10,
    imageInitial: 6,
    flippedList: true,
    rotate: 0,
    mouseMoveTimeout: 500,

    selectedGlyph: "glyphicon-chevron-up",
    DEBUG: false,
    //    DEBUG: true,
    DEBUG_CONTEXT: null,

    _loadingDiv: "#loading",
    _imagesDiv: "#images",


    init: function () {
        P.implementation.init();
    },

    debug: function (text, context) {
        P.implementation.debug(text, context);
    },

    next: function () {
        P.implementation.next();
    },

    previous: function () {
        P.implementation.previous();
    },

    select: function (item) {
        P.implementation.select(item);
    },

    loading: function (state) {
        P.implementation.loading(state)
    },

    getSelected: function () {
        return P.implementation.getSelected();
    },

    startRotation: function () {
        P.implementation.startRotation();
    },

    getCamera: function () {
        return P.implementation.getCamera();
    },

    freeze: function () {
        P.implementation.freeze();
    },

    unfreeze: function () {
        P.implementation.unfreeze();
    }
}

P.events = {
    onMouseMoveFunc: null,

    onBeforeNewSelection: function (oldItem, newItem) {
    },
    onAfterNewSelection: function (oldItem, newItem) {
        //    console.log("onAfterNewSelection " + oldItem + " " + newItem);
    },

    __onMouseMove: function () {
        if (P.events.onMouseMoveFunc != null)
            clearTimeout(P.events.onMouseMoveFunc);

        P.events.onMouseMoveFunc = setTimeout(function () {
            P.events.__onMouseMoveTimeout();
        }, P.mouseMoveTimeout);

        P.events.onMouseMove();
    },

    __onMouseMoveTimeout: function () {
        P.events.onMouseMoveTimeout();
    },

    onMouseDown: function () { },
    onMouseMove: function () { },
    onMouseUp: function () { },
    onMouseWheel: function (e) { },

    onMouseMoveTimeout: function () { },

    onTouchStart: function () { },
    onTouchMove: function () { },
    onTouchEnd: function () { },

}

P.implementation = {
    x: 0,
    y: 0,
    width: 0,
    height: 0,
    selected: this.imageInitial,
    frozen: false,
    _sensitivity: 0.4,
    _mouseDown: false,
    _selected: 0,
    _startX: 0,

    _touchhack: false,

    init: function () {
        P.mouseSensitivity = L.settings.data.mouseSensitivity;
        P.touchSensitivity = L.settings.data.touchSensitivity;
        P.flippedList = L.settings.data.flippedList;
        P.imageStart = I.imageStart;
        P.imageEnd = I.imageEnd;
        P.rotate = L.settings.data.rotate;
        P.rotateSpeed = L.settings.data.rotateSpeed;
        if (!I.flippedList)
            P.imageInitial = I.imageInitial;
        else
            P.imageInitial = (I.images + 1) - I.imageInitial;

        P.images = I.images;
        P.implementation._refreshWindowSize();

        P.select(P.imageInitial);
        P.debug("debug");

        P.ui.eventCatcher.init();
        P.ui.slider.init();
        P.ui.slideshow.init();

        W._timeout = window.setInterval(function () {
            P.ui.slider._onTimer();
        }, P.rotateSpeed);

        // register first move timeout for iPad
        if (W.touch)
            P.mouseMoveTimeout = 5000;
        P.events.onMouseMoveFunc = setTimeout(function () {
            P.events.__onMouseMoveTimeout();
            // all subsequent touches must hide hotspots sooner
            //    if (W.touch)
            //        P.mouseMoveTimeout = 1000;
        }, P.mouseMoveTimeout);

    },

    debug: function (text, context) {
        if (P.DEBUG)
            if (context == P.DEBUG_CONTEXT)
                $("#debug").html(text);
    },

    next: function () {
        P.select("+");
    },

    previous: function () {
        P.select("-");
    },

    select: function (item) {
        P.debug("select " + item)
        if (item == null)
            item = P.implementation.selected;
        if (item == "+") {
            //    P.implementation.selected++;
            //    item = P.implementation.selected;
            item = P.implementation.selected + 1;
        }
        if (item == "-") {
            //    P.implementation.selected--;
            //    item = P.implementation.selected;
            item = P.implementation.selected - 1;
        }

        if (item < P.imageStart)
            item = P.imageEnd + (item);
        else if (item > P.imageEnd)
            item = item - P.imageEnd;

        var oldItem = P.implementation.selected;

        if (P.implementation.selected != item)
            P.events.onBeforeNewSelection(P.implementation.selected, item);

        P.implementation.selected = item;
        P.ui.slideshow._showImage(item);

        if (P.implementation.selected != oldItem) {
            P.events.onAfterNewSelection(oldItem, P.implementation.selected);
            if (parent.window) {
                var message = {
                    "action": "cameraChange",
                    "value": {
                        "oldCamera": oldItem,
                        "newCamera": P.implementation.selected
                    }
                }
                // console.log("postMessage");
                // console.log(message);
                

                parent.window.postMessage(message, '*')
            }
        }
    },

    loading: function (state) {
        switch (state) {
            case true:
                P.ui.slideshow.hide();
                P.ui.loading.show();
                break;

            case false:
                P.ui.slideshow.show();
                P.ui.loading.hide();
                break;
        }
    },

    freeze: function () {
        P.implementation.frozen = true;
    },

    unfreeze: function () {
        P.implementation.frozen = false;
    },

    getCamera: function () {
        return P.implementation.selected;
    },

    startRotation: function () {
        P.ui.slideshow.startRotation();
    },

    getSelected: function () {
        return this._selected;
    },

    _setMouseDown: function (state) {
        this._mouseDown = state;
    },

    _getMouseDown: function () {
        return this._mouseDown;
    },

    _refreshWindowSize: function () {
        P.implementation.width = $(window).width();
        P.implementation.height = $(window).height();
    },

    _ontouchstart: function () {
        if (W.isZoomed())
            return false;

        //    P.ui.slideshow.delayRotation();
        P.implementation._selected = P.implementation.selected;
        P.debug("touch start " +
            P.implementation.x + "/" + P.implementation.width + " " +
            P.implementation.y + "/" + P.implementation.height)
        return false;
    },

    _ontouchmove: function () {
        if (W.isZoomed())
            return false;

        if (P.ui.slideshow._leftPressed || P.ui.slideshow._rightPressed)
            return;
        var posX = Math.round((P.implementation.x * 100) / P.implementation.width);
        var posY = Math.round((P.implementation.y * 100) / P.implementation.height);
        var relX = Math.round((P.implementation.x - P.implementation._startX) * 100 / P.implementation.width);
        var relXx = relX / (1 + P.imageStart - P.imageEnd);
        var item = Math.round(P.implementation._selected - (relXx * P.implementation._sensitivity));

        if ((posX > 15) && (posX < 85)) {
            P.select(item);
        }

        P.debug("touch move " +
            P.implementation.x + "/" + P.implementation.width + " " +
            P.implementation.y + "/" + P.implementation.height + "\n" +
            posX + " " + posY + " " + " " + relX + " " + item)
    },

    _ontouchend: function () {
        if (W.isZoomed())
            return false;

        if (P.ui.slideshow._rotate)
            P.ui.slideshow.delayRotation();
    },

    _onWheel: function (e) {
        // cross-browser wheel delta
        var e = window.event || e; // old IE support
        var delta = Math.max(- 1, Math.min(1, (e.wheelDelta || - e.deltaY || - e.detail)));
        if (delta > 0)
            P.next();
        else
            P.previous();
        P.ui.slideshow.delayRotation();
    },
}

P.ui.eventCatcher = {
    init: function () {
        $("#event-catcher").mousedown(function (e) {
            console.log(e);
            
            if (P.implementation.frozen)
                return;

            P.implementation._refreshWindowSize();
            P.implementation.x = e.clientX;
            P.implementation.y = e.clientY;
            P.implementation._sensitivity = P.mouseSensitivity;
            if (P.implementation.y > P.implementation.height - 100) {
                return;
            }
            P.implementation._startX = P.implementation.x;
            P.implementation._setMouseDown(true);
            P.implementation._ontouchstart();
            P.events.onMouseDown();
            console.log("end1")
            return false;
            // P.events.__onMouseMove();
        });
        $("#event-catcher").mousemove(function (e) {
            // console.log(e);
            if (P.implementation.frozen)
                return;

            if (P.implementation.y > P.implementation.height - 100) {
                return;
            }
            if (P.implementation._getMouseDown()) {
                P.implementation.x = e.clientX;
                P.implementation.y = e.clientY;
                P.implementation._ontouchmove();
            }
            P.events.__onMouseMove();
            P.events.onMouseMove();
            // console.log("end2")
            return false;
        });
        document.getElementById("event-catcher").onwheel = function (e) {
            console.log(e);
            if (P.implementation.frozen)
                return;

            P.ui.slideshow._onWheel(e);
            P.events.onMouseWheel(e)
            console.log("end3")
        }
        document.getElementById("event-catcher").onmousewheel = function (e) {
            console.log(e);
            if (P.implementation.frozen)
                return;

            P.ui.slideshow._onWheel(e);
            P.events.onMouseWheel(e)
            console.log("end4")
        }
        $("#event-catcher").mouseup(function (e) {
            console.log(e);
            if (P.implementation.frozen)
                return;

            P.implementation._setMouseDown(false);
            P.implementation._ontouchend();
            P.events.onMouseUp();
            console.log("end5")
        });
        document.getElementById("event-catcher").ontouchstart = function (e) {
            console.log(e);
            if (P.implementation.frozen) {
                return;
            }

            if (typeof O !== 'undefined') {
                if ((O.state) && (!W.HD)) {
                    return;
                }
            }
            // if (U.parameters.classes.indexOf("noslider") < 0) {
            //     if (P.implementation.y > (P.implementation.height - 100)) {
            //         console.log("r3")
            //         return;
            //     }
            // }
            P.implementation._refreshWindowSize();
            var touchobj = e.changedTouches[0];
            P.implementation.x = parseInt(touchobj.clientX);
            P.implementation.y = parseInt(touchobj.clientY);
            P.implementation._sensitivity = P.touchSensitivity;
            P.implementation._startX = P.implementation.x;
            P.implementation._setMouseDown(true);
            P.implementation._ontouchstart();
            P.events.onTouchStart();
            console.log("end6")
        }
        document.getElementById("event-catcher").ontouchmove = function (e) {
            console.log(e);
            if (P.implementation.frozen) {
                return;
            }

            var touchobj = e.changedTouches[0];
            P.implementation.x = parseInt(touchobj.clientX);
            P.implementation.y = parseInt(touchobj.clientY);
            P.implementation._setMouseDown(true);
            P.implementation._ontouchmove();
            P.events.onTouchMove();
            console.log("end6")
        }
        document.getElementById("event-catcher").ontouchend = function (e) {
            if (P.implementation.frozen) {
                return;
            }

            P.implementation._setMouseDown(false)
            P.implementation._ontouchend();
            P.events.__onMouseMove();
            P.events.onTouchEnd();
            console.log("end7")
        }
    },
}

P.ui.slideshow = {

    _leftPressed: false,
    _rightPressed: false,
    _rotate: false,
    _timeout: null,

    init: function () {

    },

    show: function () {
        I.show();
    },

    hide: function () {
        I.hide();
    },

    startRotation: function () {
        P.ui.slideshow._rotate = true;
        $("#rotate-hand").addClass("active");
    },

    startLeftRotation: function () {
        P.ui.slideshow.stopRotation();
        P.previous();
        this._timeout = P.ui.slider._timeout = window.setTimeout(function () {
            P.ui.slideshow._leftPressed = true;
            P.ui.slideshow._rightPressed = false;
        }, 250);
        $("#left-hand").addClass("active");
    },

    startRightRotation: function () {
        P.ui.slideshow.stopRotation();
        P.next();
        this._timeout = P.ui.slider._timeout = window.setTimeout(function () {
            P.ui.slideshow._leftPressed = false;
            P.ui.slideshow._rightPressed = true;
        }, 250);
        $("#right-hand").addClass("active");
    },

    stopRotation: function () {
        if (this._timeout != null)
            window.clearTimeout(this._timeout);
        P.ui.slideshow._leftPressed = false;
        P.ui.slideshow._rightPressed = false;
        P.ui.slideshow._rotate = false;
        $("#left-hand").removeClass("active");
        $("#right-hand").removeClass("active");
        $("#rotate-hand").removeClass("active");
    },

    delayRotation: function () {
        P.ui.slider._currentRotateDelay = P.ui.slider.rotateDelay * 1000 / L.settings.data.rotateSpeed;
        if (P.ui.slideshow._rotate == true)
            P.ui.slideshow.startRotation();
    },

    //    
    //    _showImage: function(num) {
    //        if (P.flippedList) {
    //            num = (P.images + 1) - num;
    //        }
    //        for (var i=1; i <= P.images; i++) {
    //            if (i == num) {
    //                $("#img"+i).removeClass("hidden");
    //            } else {
    //                $("#img"+i).addClass("hidden");
    //            }
    //        }
    //    },

    _showImage: function (num) {
        I.showImage(num);
    },

    _onWheel: function (e) {
        if (!$("body").hasClass("nowheel")) {
            P.implementation._onWheel(e);
        }
    },
};

P.ui.slider = {
    offset: 20,
    factor: .89,
    rotateDelay: 5,

    _currentRotateDelay: 0,
    _hrect: 0,
    _rect: 0,
    _timeout: null,

    init: function () {
        P.ui.slider._registerTouchEvents();
        P.ui.slider._registerMouseEvents();
    },

    _registerMouseEvents: function () {
        $("#rotate-hand").click(function () {
            if (!P.ui.slideshow._rotate) {
                P.ui.slideshow.startRotation();
            } else {
                P.ui.slideshow.stopRotation();
            }
        });
        if (document.getElementById("left-hand") !== null) {
            document.getElementById("left-hand").onmousedown = function () {
                P.ui.slideshow.startLeftRotation();
                return false;
            };
        }
        if (document.getElementById("right-hand") !== null) {
            document.getElementById("right-hand").onmousedown = function () {
                P.ui.slideshow.startRightRotation();
                return false;
            };
        }
        if (document.getElementById("left-hand") !== null) {
            document.getElementById("left-hand").onmouseup = function () {
                P.ui.slideshow.stopRotation();
                return false;
            };
        }
        if (document.getElementById("right-hand") !== null) {
            document.getElementById("right-hand").onmouseup = function () {
                P.ui.slideshow.stopRotation();
                return false;
            };
        }
    },

    _registerTouchEvents: function () {
        if (document.getElementById("left-hand") !== null) {
            document.getElementById("left-hand").ontouchstart = function () {
                P.ui.slideshow.startLeftRotation();
                return false;
            };
        }
        if (document.getElementById("right-hand") !== null) {
            document.getElementById("right-hand").ontouchstart = function () {
                P.ui.slideshow.startRightRotation();
                return false;
            };
        }
        if (document.getElementById("left-hand") !== null) {
            document.getElementById("left-hand").ontouchend = function () {
                P.ui.slideshow.stopRotation();
                return false;
            };
        }
        if (document.getElementById("right-hand") !== null) {
            document.getElementById("right-hand").ontouchend = function () {
                P.ui.slideshow.stopRotation();
                return false;
            };
        }
    },

    _onTimer: function () {
        if (P.ui.slideshow._leftPressed)
            P.previous();
        if (P.ui.slideshow._rightPressed)
            P.next();

        if (!P.implementation._mouseDown)
            if (P.ui.slideshow._rotate) {
                if (P.ui.slider._currentRotateDelay > 0) {
                    P.ui.slider._currentRotateDelay -= 1;
                } else {
                    P.next();
                }
            }
    },
};

P.ui.loading = {
    init: function () {

    },

    show: function () {
        $(P._loadingDiv).removeClass("hidden");
    },

    hide: function () {
        $(P._loadingDiv).addClass("hidden");
    },
}
